/*
 * (c) Copyright 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 * [See end of file]
 */

package com.hp.hpl.jena.sparql.sse.writers;

import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.hp.hpl.jena.graph.Triple;
import com.hp.hpl.jena.shared.PrefixMapping;

import com.hp.hpl.jena.sparql.ARQConstants;
import com.hp.hpl.jena.sparql.algebra.Op;
import com.hp.hpl.jena.sparql.algebra.OpPrefixesUsed;
import com.hp.hpl.jena.sparql.algebra.OpVisitor;
import com.hp.hpl.jena.sparql.algebra.op.*;
import com.hp.hpl.jena.sparql.algebra.table.TableUnit;
import com.hp.hpl.jena.sparql.core.Prologue;
import com.hp.hpl.jena.sparql.core.Quad;
import com.hp.hpl.jena.sparql.core.Var;
import com.hp.hpl.jena.sparql.core.VarExprList;
import com.hp.hpl.jena.sparql.expr.E_Aggregator;
import com.hp.hpl.jena.sparql.expr.Expr;
import com.hp.hpl.jena.sparql.expr.ExprList;
import com.hp.hpl.jena.sparql.serializer.FmtExprPrefix;
import com.hp.hpl.jena.sparql.serializer.SerializationContext;
import com.hp.hpl.jena.sparql.sse.Tags;
import com.hp.hpl.jena.sparql.util.ExprUtils;
import com.hp.hpl.jena.sparql.util.FmtUtils;
import com.hp.hpl.jena.sparql.util.IndentedWriter;
import com.hp.hpl.jena.sparql.util.NodeToLabelMap;

import com.hp.hpl.jena.query.Query;
import com.hp.hpl.jena.query.SortCondition;


// ToDo extract write of:
// Table
// Triple and quads

public class WriterOp
{
    private static final int NL = WriterLib.NL ;
    private static final int NoNL = WriterLib.NoNL ;
    private static final int NoSP = WriterLib.NoSP ;
    
    public static void out(Op op)
    { out(System.out, op) ; }
    
    public static void out(Op op, PrefixMapping pMap)
    { out(System.out, op, pMap) ; }
    
    public static void out(Op op, Prologue prologue)
    { out(System.out, op, prologue) ; }
    
    public static void out(OutputStream out, Op op)
    { out(out, op, (PrefixMapping)null) ; }

    public static void out(OutputStream out, Op op, PrefixMapping pMap)
    { out(new IndentedWriter(out), op, pMap) ; }

    public static void out(OutputStream out, Op op, Prologue prologue)
    { out(new IndentedWriter(out), op, prologue) ; }

    public static void out(IndentedWriter iWriter, Op op)
    { out(iWriter, op, (PrefixMapping)null) ; }

    public static void out(IndentedWriter iWriter, Op op, PrefixMapping pMap)
    {
        if ( pMap == null )
            pMap = OpPrefixesUsed.used(op, ARQConstants.getGlobalPrefixMap()) ;
        SerializationContext sCxt = new SerializationContext(pMap) ;
        out(iWriter, op, sCxt) ;
    }

    public static void out(IndentedWriter iWriter, Op op, Prologue prologue)
    {
        SerializationContext sCxt = new SerializationContext(prologue) ;
        out(iWriter, op, sCxt) ;
    }
    
    public static void out(OutputStream out, Op op, SerializationContext sCxt)
    {
        out(new IndentedWriter(out), op, sCxt) ;
    }

    // Actual work
    public static void out(IndentedWriter iWriter, Op op, SerializationContext sCxt)
    {
        if ( sCxt.getBNodeMap() == null )
            sCxt.setBNodeMap(new NodeToLabelMap()) ;

        boolean closeBase = false ;
        boolean closePrefix = false ;
        boolean printBase = false ;
        
        // TODO Abbreviation of URIs by base
        // Currently, does not get set except to the CWD
        if ( printBase && sCxt.getBaseIRI() != null )
        {
            WriterLib.start(iWriter, Tags.tagBase, NoNL) ;   
            iWriter.print(FmtUtils.stringForURI(sCxt.getBaseIRI())) ;
            closeBase = true ;
        }
        
        if ( sCxt.getPrefixMapping() != null )
        {
            Map m = sCxt.getPrefixMapping().getNsPrefixMap() ;
            if ( ! m.isEmpty() )
            {
                int s = iWriter.getCol() ;
                WriterLib.start(iWriter, Tags.tagPrefix, NoNL) ;
                WriterLib.start(iWriter) ;
                
                // Indent to this col.
                int len = iWriter.getCurrentOffset() ;
                
                iWriter.incIndent(len) ;
                Iterator iter = m.keySet().iterator();
                boolean first = true ;
                for ( ; iter.hasNext() ; )
                {
                    if ( ! first )
                        iWriter.println() ;
                    first = false ;
                    String prefix = (String)iter.next();
                    String uri = sCxt.getPrefixMapping().getNsPrefixURI(prefix) ;
                    WriterLib.start(iWriter) ;
                    iWriter.print(prefix) ;
                    iWriter.print(": ") ;
                    iWriter.print("<"+uri+">") ;
                    WriterLib.finish(iWriter) ;
                }
                iWriter.decIndent(len) ;
                WriterLib.finish(iWriter) ;
                
                iWriter.ensureStartOfLine() ;
                closePrefix = true ;
            }
        }
        
        op.visit(new OpWriterWorker(iWriter, sCxt)) ;
        if ( closeBase )
            WriterLib.finish(iWriter, Tags.tagBase) ;
        if ( closePrefix )
            WriterLib.finish(iWriter, Tags.tagPrefix) ;
        iWriter.ensureStartOfLine() ;
        iWriter.flush();
    }

    
    private static class OpWriterWorker implements OpVisitor
    {
        private IndentedWriter out ;
        private SerializationContext sContext ;
//        private VarAlloc varAlloc = new VarAlloc("__") ;
        private FmtExprPrefix fmtExpr ; 
        
        public OpWriterWorker(IndentedWriter out, SerializationContext sCxt)
        { 
            this.sContext = sCxt ;
            this.out = out ;
            this.fmtExpr = new FmtExprPrefix(out, sCxt) ;
        }
        
        private void visitOp2(Op2 op, ExprList exprs)
        {
            start(op, NL) ;
            printOp(op.getLeft()) ;

            //out.println() ; 
            out.ensureStartOfLine() ;

            printOp(op.getRight()) ;

            out.ensureStartOfLine() ;
            //out.println() ; 

            if ( exprs != null )
                ExprUtils.fmtPrefix(out, exprs) ;
            finish(op) ;
        }

        private void visitOp1(Op1 op)
        {
            start(op, NL) ;
            printOp(op.getSubOp()) ;
            finish(op) ;
        }

        public void visit(OpQuadPattern opQuadP)
        { 
            if ( opQuadP.getQuads().size() == 1 )
            {
                start(opQuadP, NoNL) ;
                formatQuad((Quad)opQuadP.getQuads().get(0)) ;
                finish(opQuadP) ;
                return ;
            }
            start(opQuadP, NL) ;
            for ( Iterator iter = opQuadP.getQuads().listIterator() ; iter.hasNext() ;)
            {
               Quad quad = (Quad)iter.next() ;
               formatQuad(quad) ;
               out.println() ;
            }
            finish(opQuadP) ;
        }
        
        public void visit(OpBGP opBGP)
        {
            if ( opBGP.getPattern().size() == 1 )
            {
                start(opBGP, NoNL) ;
                formatTriple(opBGP.getPattern().get(0)) ;
                finish(opBGP) ;
                return ;
            }
            
            start(opBGP, NL) ;
            for ( Iterator iter = opBGP.getPattern().iterator() ; iter.hasNext() ;)
            {
               Triple t = (Triple)iter.next() ;
               formatTriple(t) ;
               out.println() ;
            }
            finish(opBGP) ;
        }
        
        public void visit(OpJoin opJoin)
        { visitOp2(opJoin, null) ; }

        public void visit(OpLeftJoin opLeftJoin)
        { visitOp2(opLeftJoin, opLeftJoin.getExprs()) ; }
        
        public void visit(OpDiff opDiff)
        { visitOp2(opDiff, null) ; }
    
        public void visit(OpUnion opUnion)
        { visitOp2(opUnion, null) ; } 
    
        public void visit(OpFilter opFilter)
        { 
            start(opFilter, NoNL) ;
            ExprList exprs = opFilter.getExprs() ;
            if ( exprs == null )
            { start() ; finish() ; }
            else
                ExprUtils.fmtPrefix(out, exprs, sContext.getPrefixMapping()) ;
            out.println();
            printOp(opFilter.getSubOp()) ;
            finish(opFilter) ;
        }
    
        public void visit(OpGraph opGraph)
        {
            start(opGraph, NoNL) ;
            out.println(FmtUtils.stringForNode(opGraph.getNode())) ;
            out.incIndent() ;
            opGraph.getSubOp().visit(this) ;
            finish(opGraph) ;
        }

        public void visit(OpService opService)
        {
            start(opService, NoNL) ;
            out.println(FmtUtils.stringForNode(opService.getService())) ;
            out.incIndent() ;
            opService.getSubOp().visit(this) ;
            finish(opService) ;
        }

        public void visit(OpTable opTable)
        {
            if ( TableUnit.isJoinUnit(opTable.getTable()) )
            {
                start(opTable, NoNL) ;
                out.print("unit") ;
                finish(opTable) ;
                return ;
            }
            
            start(opTable, NL) ;
            WriterTable.outNoTag(out, opTable.getTable(), sContext);
            finish(opTable) ;
        }

        public void visit(OpDatasetNames dsNames)
        {
            start(dsNames, NoNL) ;
            WriterNode.out(out, dsNames.getGraphNode(), sContext) ;
            finish(dsNames) ;
        }

        public void visit(OpExt opExt)
        {
            //start(opExt, NL) ;
            opExt.output(out, sContext) ;
            //finish(opExt) ;
        }

        public void visit(OpNull opNull)
        { start(opNull, NoSP) ; finish(opNull) ; } 

        public void visit(OpList opList)
        {
            visitOp1(opList) ;
        }
        
        public void visit(OpGroupAgg opGroup)
        {
            start(opGroup, NoNL) ;
            writeNamedExprList(opGroup.getGroupVars()) ;
            if ( ! opGroup.getAggregators().isEmpty() )
            {
                // --- Aggregators
                out.print(" ") ;
                start() ;
                out.incIndent() ;
                boolean first = true ;
                for ( Iterator iter = opGroup.getAggregators().iterator() ; iter.hasNext() ; )
                {
                    if ( ! first )
                        out.print(" ") ;
                    first = false ;
                    E_Aggregator agg = (E_Aggregator)iter.next();
                    Var v = agg.asVar() ;
                    String str = agg.getAggregator().toPrefixString() ;
                    start() ;
                    out.print(v) ;
                    out.print(" ") ;
                    out.print(str) ;
                    finish() ;
                }
                finish() ;
                out.decIndent() ;
            }
            out.println() ;
            printOp(opGroup.getSubOp()) ;
            finish(opGroup) ;
        }
        
        public void visit(OpOrder opOrder)
        { 
            start(opOrder, NoNL) ;
            
            // Write conditions
            start() ;

            boolean first = true ;
            for ( Iterator iter = opOrder.getConditions().iterator() ; iter.hasNext(); )
            {
                if ( ! first )
                    out.print(" ") ;
                first = false ;
                SortCondition sc = (SortCondition)iter.next() ;
                formatSortCondition(sc) ;
            }
            finish() ;
            printOp(opOrder.getSubOp()) ;
            finish(opOrder) ;
        }
        
        
        // Neater would be a pair of explicit SortCondition formatter
        public void formatSortCondition(SortCondition sc)
        {
            boolean close = true ;
            String tag = null ;
            
            if ( sc.getDirection() != Query.ORDER_DEFAULT ) 
            {            
                if ( sc.getDirection() == Query.ORDER_ASCENDING )
                {
                    tag = Tags.tagAsc ; 
                    WriterLib.start(out, tag, NoNL) ;
                }
            
                if ( sc.getDirection() == Query.ORDER_DESCENDING )
                {
                    tag = Tags.tagDesc ; 
                    WriterLib.start(out, tag, NoNL) ;
                }
                
            }
            
            fmtExpr.format(sc.getExpression()) ;
            
            if ( tag != null )
                WriterLib.finish(out, tag) ;
        }



        public void visit(OpProject opProject)
        { 
            start(opProject, NoNL) ;
            writeVarList(opProject.getVars()) ;
            out.println();
            printOp(opProject.getSubOp()) ;
            finish(opProject) ;
        }

        public void visit(OpDistinct opDistinct)
        {
            visitOp1(opDistinct) ;
        }
        
        public void visit(OpReduced opReduced)
        {
            visitOp1(opReduced) ;
        }
        
        public void visit(OpAssign opAssign)
        {
            start(opAssign, NoNL) ;
            writeNamedExprList(opAssign.getVarExprList()) ;
            out.println();
            printOp(opAssign.getSubOp()) ;
            finish(opAssign) ;
        }
        
        public void visit(OpSlice opSlice)
        { 
            start(opSlice, NoNL) ;
            writeIntOrDefault(opSlice.getStart()) ;
            out.print(" ") ;
            writeIntOrDefault(opSlice.getLength()) ;
            out.println() ;
            printOp(opSlice.getSubOp()) ;
            finish(opSlice) ;
        }

        private void writeIntOrDefault(long value)
        {
            String x = "_" ;
            if ( value != Query.NOLIMIT )
                x = Long.toString(value) ;
            out.print(x) ;
        }
            
        private void start(Op op, int newline)
        {
            WriterLib.start(out, op.getName(), newline) ;
        }
        
        private void finish(Op op)
        {
            WriterLib.finish(out, op.getName()) ;
        }

        private void start()    { WriterLib.start(out) ; }
        private void finish()   { WriterLib.finish(out) ; }

        
        private void printOp(Op op)
        {
            if ( op == null )
            {
                WriterLib.start(out, Tags.tagNull, NoSP) ;
                WriterLib.finish(out, Tags.tagNull) ;
            }
            else
                op.visit(this) ;
        }
        
        private void writeVarList(List vars)
        {
            start() ;
            boolean first = true ;
            for ( Iterator iter = vars.iterator() ; iter.hasNext() ; )
            {
                if ( ! first )
                  out.print(" ") ;
                first = false ;
                Var v = (Var)iter.next() ;
                out.print(v.toString()) ;
            }
            finish() ;
        }
        
        private void writeNamedExprList(VarExprList project)
        {
            start() ;
            boolean first = true ;
            for ( Iterator iter = project.getVars().iterator() ; iter.hasNext() ; )
            {
                if ( ! first )
                  out.print(" ") ;
                first = false ;
                Var v = (Var)iter.next() ;
                Expr expr = project.getExpr(v) ;
                if ( expr != null )
                {
                    start() ;
                    out.print(v.toString()) ;
                    out.print(" ") ;
                    ExprUtils.fmtPrefix(out, expr, sContext.getPrefixMapping()) ;
                    finish() ;
                }
                else
                    out.print(v.toString()) ;
            }
            finish() ;
        }


        private void formatTriple(Triple tp)
        { WriterNode.out(out, tp, sContext) ; }
        
        private void formatQuad(Quad qp)
        { WriterNode.out(out, qp, sContext) ; }
        
//        private void formatTriple(Triple tp)
//        {
//            WriterLib.start(out, Tags.tagTriple, NoNL) ;
//            WriterNode.out(out, tp.getSubject(), sContext) ;
//            out.print(" ") ;
//            WriterNode.out(out, tp.getPredicate(), sContext) ;
//            out.print(" ") ;
//            WriterNode.out(out, tp.getObject(), sContext) ;
//            WriterLib.finish(out, Tags.tagTriple) ;
//        }
//
//        private void formatQuad(Quad qp)
//        {
//            WriterLib.start(out, Tags.tagQuad, NoNL) ;
//            WriterNode.out(out, qp.getGraph(), sContext) ;
//            out.print(" ") ;
//            WriterNode.out(out, qp.getSubject(), sContext) ;
//            out.print(" ") ;
//            WriterNode.out(out, qp.getPredicate(), sContext) ;
//            out.print(" ") ;
//            WriterNode.out(out, qp.getObject(), sContext) ;
//            WriterLib.finish(out, Tags.tagQuad) ;
//        }
    }
}

/*
 * (c) Copyright 2006, 2007 Hewlett-Packard Development Company, LP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */